/*
 *
 * Name:	Plugin_procs.c
 * 
 * Description:
 * 
 * Change Log:
 *
 ********************************************************************
 *                                                                  *
 * (C) Copyright InSoft, Inc. 1995 All rights reserved              *
 *                                                                  *
 * You have a royalty-free right to use, modify, and reproduce the  *
 * the Sample Files (and/or any modified version) in any way you    *
 * find useful, provided that you agree that InSoft has no warranty *
 * obligations or liability for any Sample OpenDVE Plugin Files     *
 * which are modified.                                              *
 *                                                                  *
 ********************************************************************
 */

#include <stdio.h>
#include <sys/types.h>

#include "ISSamplePluginRaw.h"

#include <dve/dve.h>
#include <dve/clipboard.h>
#include "SamplePluginProtocol.h"

extern samplePluginMessageStructure *rawmsg;

/*
 * Establish available drinks
 */
char* drinknames[] = {
   "Coke",
   "Pepsi",
   "Beer",
   "Tequila",
   "Water"
};

/*
 * Establish available music selections
 */
char *musicchoices[] = {
   "None",
   "Rock",
   "Blues",
   "Rock and Blues",
   "Country",
   "Rock and Country",
   "Blues and Country",
   "Rock, Blues, and Country"
};

extern DveSession dveSession;
extern DveClipboard clipboard;

/* 
 * Forward declarations
 */
void InsertNewTextMessage(DveUserHandle sender, char *text, int size);
void SendCurrentStatus(u_int32 messageType);
void QueryUserForCurrentStatus(DveUserHandle member);

void	
ProcessClipboardData(char *format, char *data, int length)
{
   fprintf(stderr, "Clipboard contents:\n");
   fprintf(stderr, "%s", data);
}

void 
CleanupPlugin(void)
{
   ShutdownOpenDVEClipboard(clipboard);
}

void 
QueryPluginStatus(DveConfParticipant *part)
{
   fprintf(stderr, "We should ask Plugin status from %s@%s (%s)\n",
                    part->userName, part->hostName,
                    inet_ntoa(*(struct in_addr*)&((part->addr.v3))));
   QueryUserForCurrentStatus(part->userHandle);
}

void 
MemberLeft(DveConfParticipant member)
{
   fprintf(stderr, "User Left the Conference - %s@%s (%s)\n",
                    member.userName, member.hostName,
                    inet_ntoa(*(struct in_addr*)&((member.addr.v3))));
}

void 
MemberAdded(DveConfParticipant member)
{
   fprintf(stderr, "User Added to Conference - %s@%s (%s)\n",
                    member.userName, member.hostName,
                    inet_ntoa(*(struct in_addr*)&((member.addr.v3))));
}

void 
ProcessPluginMessage(u_int32 messageType, void *data, DveUserHandle sender) 
{
   samplePluginMessageStructure message;
   char *tempPointer;

   memcpy(&message, data, sizeof(samplePluginMessageStructure));

   switch (messageType){
      case TEMPERATURE_CHANGE:
         fprintf(stderr, "Temperature: %d\n", ntohl(message.temperature));
         rawmsg->temperature = ntohl(message.temperature);
         break;
      case MUSIC_CHANGE:
         fprintf(stderr, "Music Choices: %s\n", (char *)musicchoices[ntohl(message.musicChoices)]);
         rawmsg->musicChoices = ntohl(message.musicChoices);
         break;
      case DRINK_CHANGE:
         fprintf(stderr, "Drink Selection: %s\n", (char *)drinknames[ntohl(message.drinkSelection)]);
         rawmsg->drinkSelection = ntohl(message.drinkSelection);
         break;
      case TEXT_MESSAGE:
         tempPointer = ((samplePluginMessageStructure *)data)->message;
         InsertNewTextMessage(sender, tempPointer, ntohl(message.messageSize));
         break;
      case UPDATE_ALL_DATA:
         fprintf(stderr, "Temperature: %d\n", ntohl(message.temperature));
         fprintf(stderr, "Music Choices: %s\n", (char *)musicchoices[ntohl(message.musicChoices)]);
         fprintf(stderr, "Drink Selection: %s\n", (char *)drinknames[ntohl(message.drinkSelection)]);
         rawmsg->musicChoices = ntohl(message.musicChoices);
         rawmsg->drinkSelection = ntohl(message.drinkSelection);
         rawmsg->temperature = ntohl(message.temperature);
         break;
      case QUERY_PLUGIN_STATUS:
         fprintf(stderr, "Received a request for status...\n");
         SendCurrentStatus(UPDATE_ALL_DATA);
         fprintf(stderr, "...sent current status of Plugin.\n");
         break;
   }
}

void
SendCurrentStatus(u_int32 messageType)
{
   samplePluginMessageStructure *msg;
 
   msg = (samplePluginMessageStructure *) dveMallocDataPacket(dveSession,
                                          sizeof(samplePluginMessageStructure));
   if (!msg)
      return;
 
   msg->protocolVersion = htonl(VERSION);
 
   /*
    * Get the settings.
    */
   msg->temperature = htonl(rawmsg->temperature);
   msg->musicChoices = htonl(rawmsg->musicChoices);
   msg->drinkSelection = htonl(rawmsg->drinkSelection);
   msg->messageSize = 0;

   (void)SendPluginMessage(dveSession,
                           SAMPLE_PLUGIN_PORT, messageType, msg,
                           sizeof(samplePluginMessageStructure));
   return;
}

void 	
QueryUserForCurrentStatus(DveUserHandle member)
{
   samplePluginMessageStructure    *msg;
 
   msg = (samplePluginMessageStructure *) dveMallocDataPacket(dveSession,
                                                              sizeof(samplePluginMessageStructure));
   if (!msg)
      return;
 
   msg->protocolVersion = VERSION;
   msg->temperature = 0;
   msg->musicChoices = 0;
   msg->drinkSelection = 0;
   msg->messageSize = 0;
 
   SendPluginMessageToMember(dveSession, SAMPLE_PLUGIN_PORT,
                             member, QUERY_PLUGIN_STATUS, msg,
                             sizeof(samplePluginMessageStructure));
   return;
}

int
SendTextMessage(char *txtMessage, int length)
{
   samplePluginMessageStructure *msg;
   int rc;
 
   msg = (samplePluginMessageStructure *) dveMallocDataPacket(dveSession,
                                          sizeof(samplePluginMessageStructure) + length);
   if (!msg)
      return(-1);
 
   msg->protocolVersion = VERSION;
   msg->temperature = 0;
   msg->musicChoices = 0;
   msg->drinkSelection = 0;
   msg->messageSize = htonl(length);

   memcpy(msg->message, txtMessage, length);
 
   rc = SendPluginMessage(dveSession,
                          SAMPLE_PLUGIN_PORT, TEXT_MESSAGE, msg,
                          sizeof(samplePluginMessageStructure) + length);
 
   if (rc == 0)
      InsertNewTextMessage(0, txtMessage, length);
   return(rc);
}

void
InsertNewTextMessage(DveUserHandle sender, char *text, int size)
{
   DveConfParticipant member;
   char newMessage[1024];
   char fullName[256];
 
   if (sender == 0)
   {
      sprintf(newMessage, "Message from: ME!");
   }
   else if (dveGetOneMember(dveSession, sender, &member) < 0)
   {
      sprintf(newMessage, "Message From: Unknown User");
   }
   else
   {
      sprintf(newMessage, "Message From: %s@%s", member.userName, member.hostName);
   }
   fprintf(stderr, "%s\n", newMessage);
   fprintf(stderr, "%s\n", text);
}

int
DisplayMainMenu() 
{
   fprintf(stderr, "\nMenu selection?\n{1} Temperature\n{2} Music\n{3} Drink\n"
                   "{4} Copy to Clipboard\n{5} Show Clipboard\n{6} Send Message\n"
                   "{7} Display All\n");
   return(MAIN_STATE);
}

int  
ProcessInput(char *buf, int new_state)
{
   int itmp;

   switch(new_state)
   {
      case MAIN_STATE:
         switch(buf[0])
         {
            case '1':
               fprintf(stderr, "New temperature?\n");
               new_state=TEMPERATURE_STATE;
               break;
            case '2':
               fprintf(stderr, "Music choice?\n{1} None\n{2} Rock\n{3} Blues\n"
                               "{4} Rock and Blues\n{5} Country\n{6} Rock and Country\n"
                               "{7} Blues and Country\n{8} Rock, Blues, and Country\n");
               new_state=MUSIC_STATE;
               break;
            case '3':
               fprintf(stderr, "Drink selection?\n{1} Coke\n{2} Pepsi\n{3} Beer\n"
                               "{4} Tequila\n{5} Water\n");
               new_state=DRINK_STATE;
               break;
            case '4':
               fprintf(stderr, "Message for clipboard?\n");
               new_state=COPYTOCLIP_STATE;
               break;
            case '5':
               if (GetOpenDVEClipboard(clipboard, IS_TEXT_CLIPBOARD_ITEM) < 0)
                  fprintf(stderr, "Clipboard empty!\n\7");
               new_state=DisplayMainMenu();
               break;
            case '6':
               fprintf(stderr, "New message?\n");
               new_state=SENDMSG_STATE;
               break;
            case '7':
               fprintf(stderr, "Temperature: %d\n", rawmsg->temperature);
               fprintf(stderr, "Music Choices: %s\n", musicchoices[rawmsg->musicChoices]);
               fprintf(stderr, "Drink Selection: %s\n", drinknames[rawmsg->drinkSelection]);
               if (GetOpenDVEClipboard(clipboard, IS_TEXT_CLIPBOARD_ITEM) < 0)
                  fprintf(stderr, "Clipboard empty!\n\7");
               new_state=DisplayMainMenu();
               break;
            default:
               fprintf(stderr, "Invalid selection.\n\7");
               new_state=DisplayMainMenu();
               break;
         }
         break;
      case TEMPERATURE_STATE:
         itmp = atoi(buf);
         if (((itmp < 0) || (itmp > 100)) ||
             ((itmp == 0) && (buf[0] != '0')))
         {
            fprintf(stderr, "Invalid temperature.\n\7");
         }
         else
         {
            rawmsg->temperature = itmp;
            SendCurrentStatus(TEMPERATURE_CHANGE);
         }
         new_state=DisplayMainMenu();
         break;
      case MUSIC_STATE:
         itmp = atoi(buf);
         if ((itmp < 1) || (itmp > 8))
         {
            fprintf(stderr, "Invalid selection.\n\7");
         }
         else
         {
            rawmsg->musicChoices = (itmp - 1);
            SendCurrentStatus(MUSIC_CHANGE);
         }
         new_state=DisplayMainMenu();
         break;
      case DRINK_STATE:
         itmp = atoi(buf);
         if ((itmp < 1) || (itmp > 5))
         {
            fprintf(stderr, "Invalid selection.\n\7");
         }
         else
         {
            rawmsg->drinkSelection = (itmp - 1);
            SendCurrentStatus(DRINK_CHANGE);
         }
         new_state=DisplayMainMenu();
         break;
      case COPYTOCLIP_STATE:
         new_state=DisplayMainMenu();
         if (CopyDataToClipboard(clipboard, buf, strlen(buf) + 1, IS_TEXT_CLIPBOARD_ITEM) < 0)
         {
            fprintf(stderr, "Error copying to clipboard.\n\7");
         }
         else
         {
            fprintf(stderr, "Copy complete.\n");
         } 
         break;
      case SENDMSG_STATE:
         if (SendTextMessage(buf, strlen(buf)) < 0)
         {
            fprintf(stderr, "Error sending message.\n\7");
         }
         else
         {
            fprintf(stderr, "Message sent.\n");
            new_state=DisplayMainMenu();
            break;
         }
      default:
         break;
   }
   return(new_state);
}

void 
ShowPlugin(void)
{
   fprintf(stderr, "ShowPlugin\n");
}

void 
HidePlugin(void)
{
   fprintf(stderr, "HidePlugin\n");
}
